#include "Field.h"
#include <iostream>
#include <algorithm>
#include "Result.h"
using namespace std;
Field::Field(Result& result)
	:Object(),animals(),animalCount(0),workMatrix(nullptr),result(result),maxSafety(0)
{
}
istream& operator>>(istream& in, Field& field)
{
	int rows,cols,animals;
	in>>rows>>cols>>animals;
	field.createMatrix(rows,cols);
	field.animals.resize(animals);
	field.sortedAnimals.resize(animals);
	field.readMatrix(in);
	for(int i = 0;i<animals; i++)
	{
		in>>field.animals[i];
		field.sortedAnimals[i] = &field.animals[i];
	}
	field.animalCount = animals;
	return in;
}
void Field::sortByEffectOnField()
{
	sort(sortedAnimals.begin(),sortedAnimals.end(),[=](const Animal* animal1,const Animal* animal2)
	{
		return animal1->getEffectOnField()<animal2->getEffectOnField();
	});
}

void Field::startJumps()
{
	int safety = 0;
	createWorkMatrix();
	for(int i = 0; i < getAnimalCount(); i++)
	{
		const Animal* animal = sortedAnimals[i];
		safety+=makeTheSafestJump(animal);
	}
	destroyWorkMatrix();
	if(maxSafety<safety)
	{
		maxSafety = safety;
		result.setWorkAsCurrent();
	}
}
int Field::makeTheSafestJump(const Animal* animal)
{
	int safety = 0;
	int rows = rowCount - animal->getRowsCount();
	int cols = columnCount - animal->getColumnsCount();
	int row =-1, col=-1;
	for( int r = 0; r < rows; ++r ) 
		for( int c = 0; c < cols; ++c)
		{
			int safeIndex = calcSafetyIndex(*animal,r,c);
			if(safeIndex>safety)
			{
				row = r;
				col = c;
				safety = safeIndex;
			}
		}
	if(row!=-1)
		makeJump(*animal,row,col);
	return safety;
}

int Field::calcSafetyIndex(const Animal& animal, int row, int col)const 
{
	if(animal.getRowsCount()+row >getRowsCount()||
		animal.getColumnsCount()+ col > getColumnsCount())
			throw exception("Out of the field");
	int res = 0;
	for(int r = 0; r < animal.getRowsCount(); r++ )
		for(int c = 0; c < animal.getColumnsCount(); c++)
			if(animal[r][c] == 1)
			{
				int safety = workMatrix[row+r][col+c];
				if(safety<animal.getSafetyLimit())
					return 0;
				else 
					res += safety;
			}
	return res;

}

void Field::makeJump(const Animal& animal, int row, int col)
{
	int animalRows = animal.getRowsCount();
	int animalCols = animal.getColumnsCount();
	int actionIndex = animal.getActionIndex();
	for(int r = 0; r < animalRows; r++ )
		for(int c = 0; c < animalCols; c++)
			if(animal[r][c]!=0)
				workMatrix[r+row][c+col]/=actionIndex;
	result.addJump(animal.getId(), row, col);

}
void Field::createWorkMatrix()
{
	workMatrix = new int*[rowCount];
	for(int row = 0; row<rowCount; row++)
	{
		workMatrix[row] = new int[columnCount];
		for(int col = 0; col<columnCount; col++)
		{
			workMatrix[row][col] = matrix[row][col];
		}
	}

}
void Field::destroyWorkMatrix()
{
	if(workMatrix == nullptr)
		return;
	for(int i = 0; i<rowCount; i++)
		delete[] workMatrix[i];
	delete[] workMatrix;
	workMatrix = nullptr;
}
Field::~Field(void)
{
	destroyWorkMatrix();
}
